@@ -13,20 +13,28 @@ Various utilities
    ~ExcelDatabaseFileBase
    ~ExcelDatabaseFileGeneric
    ~ExcelReadError
+   ~findbyname
+   ~findbypv
+   ~findCatalogsInNamespace
    ~full_dotted_name
+   ~getCatalog
    ~getDatabase
+   ~getDefaultCatalog
    ~getDefaultDatabase
+   ~getDefaultNamespace
    ~ipython_profile_name
    ~itemizer
    ~json_export
    ~json_import
    ~listdevice
    ~listobjects
+   ~ListRuns
    ~listruns
+   ~listruns_v1_4
    ~object_explorer
    ~pairwise
-   ~print_snapshot_list
    ~print_RE_md
+   ~print_snapshot_list
    ~quantify_md_key_use
    ~redefine_motor_position
    ~replay
@@ -60,6 +68,7 @@ from bluesky import plan_stubs as bps
 from bluesky.callbacks.best_effort import BestEffortCallback
 from collections import defaultdict
 from collections import OrderedDict
+from dataclasses import dataclass
 from email.mime.text import MIMEText
 from event_model import NumpyEncoder
 import databroker
@@ -72,6 +81,7 @@ import openpyxl
 import openpyxl.utils.exceptions
 import ophyd
 import os
+import pandas as pd
 import psutil
 import pyRestTable
 import re
@@ -92,7 +102,11 @@ MAX_EPICS_STRINGOUT_LENGTH = 40
 
 
 class ExcelReadError(openpyxl.utils.exceptions.InvalidFileException):
-    ...
+    """
+    Exception when reading Excel spreadsheet.
+
+    .. index:: apstools Exception; ExcelReadError
+    """
 
 
 def cleanupText(text):
@@ -142,7 +156,8 @@ def device_read2table(
     """
     warnings.warn(
         "DEPRECATED: device_read2table() will be removed"
-        " in a future release.  Use listdevice() instead."
+        " in a future release.  Use listdevice() instead.",
+        DeprecationWarning
     )
     listdevice(
         device,
@@ -380,7 +395,380 @@ def itemizer(fmt, items):
     return [fmt % k for k in items]
 
 
+def getCatalog(ref=None):
+    if isinstance(ref, str):  # and ref in databroker.catalog:
+        return databroker.catalog[ref]
+    if ref is not None and hasattr(ref, "v2"):
+        return ref.v2
+
+    cat = getDefaultCatalog()
+    if cat is None:
+        raise ValueError("Cannot identify default databroker catalog.")
+    return cat
+
+
+def findCatalogsInNamespace():
+    g = {}
+    g.update(getDefaultNamespace())
+    ns_cats = {}
+    for k, v in g.items():
+        if not k.startswith("_") and hasattr(v, "__class__"):
+            try:
+                if (hasattr(v.v2, "container") and hasattr(v.v2, "metadata")):
+                    ns_cats[k] = v
+            except (AttributeError, TypeError):
+                continue
+    return ns_cats
+
+
+def getDefaultCatalog():
+    cats = findCatalogsInNamespace()
+    if len(cats) == 1:
+        return cats[list(cats.keys())[0]]
+    if len(cats) > 1:
+        choices = '   '.join([
+            f"{k} ({v.name})"
+            for k, v in cats.items()
+        ])
+        raise ValueError(
+            "No catalog defined.  "
+            "Multiple catalog objects available."
+            "  Specify one of these:"
+            f" {choices}"
+        )
+
+    cats = list(databroker.catalog)
+    if len(cats) == 1:
+        return databroker.catalog[cats[0]]
+    if len(cats) > 1:
+        choices = '   '.join([f'databroker.catalog[\"{k}\"]' for k in cats])
+        raise ValueError(
+            "No catalog defined.  "
+            "Multiple catalog configurations available."
+            "  Create specific catalog object from one of these commands:"
+            f" {choices}"
+        )
+
+    raise ValueError("No catalogs available.")
+
+
+def getDefaultNamespace():
+    """
+    get the IPython shell's namespace dictionary (or globals() if not found)
+    """
+    try:
+        from IPython import get_ipython
+
+        ns = get_ipython().user_ns
+    except (ModuleNotFoundError, AttributeError):
+        ns = globals()
+    return ns
+
+
+@dataclass
+class ListRuns:
+    """
+    List the runs from the given catalog according to some options.
+
+    EXAMPLE::
+
+        ListRuns(cat).to_dataframe()
+
+    PUBLIC METHODS
+
+    .. autosummary::
+
+        ~to_dataframe
+        ~to_table
+        ~parse_runs
+
+    INTERNAL METHODS
+
+    .. autosummary::
+
+        ~_get_by_key
+        ~_check_cat
+        ~_apply_search_filters
+        ~_sorter
+        ~_check_keys
+
+    """
+
+    cat: object = None
+    query: dict = None
+    keys: str = None
+    missing: str = ""
+    num: int = 20
+    reverse: bool = True
+    since: str = None
+    sortby: str = "time"
+    timefmt: str = "%Y-%m-%d %H:%M:%S"
+    until: str = None
+
+    _default_keys = "scan_id time plan_name detectors"
+
+    def _get_by_key(self, md, key):
+        """
+        Get run's metadata value by key.
+
+        Look in ``start`` document first.
+        If not found, look in ``stop`` document.
+        If not found, report using ``self.missing``.
+
+        If ``key`` is found but value is None, report as ``self.missing``.
+
+        The ``time`` key will be formatted by the ``self.timefmt`` value.
+        See https://strftime.org/ for examples.  The special ``timefmt="raw"``
+        is used to report time as the raw value (floating point time as used in
+        python's ``time.time()``).
+
+        A special syntax of ``key`` allows reporting of keys in other
+        metadata subdictionaries.  The syntax is ``doc.key`` (such as
+        specifying the ``time`` key  from the ``stop`` document:
+        ``stop.time``) where a ``.`` is used to separate the
+        subdictionary name (``doc``) from the ``key``.
+        **Note**:
+        It is not possible to ``sortby`` the dotted-key syntax
+        at this time.
+        """
+        v = None
+        if key == "time":
+            v = md["start"][key]
+            if self.timefmt != "raw":
+                ts = datetime.datetime.fromtimestamp(v)
+                v = ts.strftime(self.timefmt)
+        elif key in md["start"]:
+            v = md["start"].get(key, self.missing)
+        elif md["stop"] and key in md["stop"]:
+            v = md["stop"].get(key, self.missing)
+        elif len(key.split(".")) == 2:
+            # dotted-key syntax
+            doc, key = key.split(".")
+            if md[doc] is not None:
+                v = md[doc].get(key, self.missing)
+                if key == "time" and self.timefmt != "raw":
+                    ts = datetime.datetime.fromtimestamp(v)
+                    v = ts.strftime(self.timefmt)
+        return v or self.missing
+
+    def _check_cat(self):
+        if self.cat is None:
+            self.cat = getCatalog()
+
+    def _apply_search_filters(self):
+        since = self.since or "1995-01-01"
+        until = self.until or "2100-12-31"
+        self._check_cat()
+        query = {}
+        query.update(databroker.queries.TimeRange(since=since, until=until))
+        query.update(self.query or {})
+        cat = self.cat.v2.search(query)
+        return cat
+
+    def parse_runs(self):
+        """Parse the runs for the given metadata keys.  Return a dict."""
+        self._check_cat()
+        self._check_keys()
+        cat = self._apply_search_filters()
+        num_runs_requested = min(abs(self.num), len(cat))
+        dd = {
+            key: [
+                self._get_by_key(run.metadata, key)
+                for _, run in sorted(
+                    cat.items(), key=self._sorter, reverse=self.reverse
+                )[:num_runs_requested]
+            ]
+            for key in self.keys
+        }
+        return dd
+
+    def _sorter(self, args):
+        """Sort runs in desired order based on metadata key."""
+        # args : (uid, run)
+        md = args[1].metadata
+        for doc in "start stop".split():
+            if md[doc] and self.sortby in md[doc]:
+                return md[doc][self.sortby] or self.missing
+        return self.missing
+
+    def _check_keys(self):
+        """Check that self.keys is a list of strings."""
+        self.keys = self.keys or self._default_keys
+        if isinstance(self.keys, str) and self.keys.find(" ") >= 0:
+            # convert a space-delimited string of names
+            self.keys = self.keys.split()
+
+    def to_dataframe(self):
+        """Output as pandas DataFrame object"""
+        self._check_keys()
+        dd = self.parse_runs()
+        return pd.DataFrame(dd, columns=self.keys)
+
+    def to_table(self, fmt=None):
+        """Output as pyRestTable object."""
+        self._check_keys()
+        dd = self.parse_runs()
+
+        table = pyRestTable.Table()
+        rows = []
+        for label, values in dd.items():
+            table.addLabel(label)
+            rows.append(values)
+        table.rows = list(zip(*rows))
+
+        return table.reST(fmt=fmt or "simple")
+
+
 def listruns(
+    cat=None,
+    keys=None,
+    missing="",
+    num=20,
+    printing="smart",
+    reverse=True,
+    since=None,
+    sortby="time",
+    tablefmt="dataframe",
+    timefmt="%Y-%m-%d %H:%M:%S",
+    until=None,
+    **query
+):
+    """
+    List runs from catalog.
+
+    This function provides a thin interface to the highly-reconfigurable
+    ``ListRuns()`` class in this package.
+
+    For the old version of this function, see
+    ``listruns_v1_4()``.  That code was last
+    updated in apstools version 1.4.1.  It
+    will be removed some time in the future.
+
+    PARAMETERS
+
+    cat
+        *object* :
+        Instance of databroker v1 or v2 catalog.
+    keys
+        *str* or *[str]* or None:
+        Include these additional keys from the start document.
+        (default: ``None`` means ``"scan_id time plan_name detectors"``)
+    missing
+        *str*:
+        Test to report when a value is not available.
+        (default: ``""``)
+    num
+        *int* :
+        Make the table include the ``num`` most recent runs.
+        (default: ``20``)
+    printing
+        *bool* or ``"smart"``:
+        If ``True``, print the table to stdout.
+        If ``"smart"``, then act as shown below.
+        (default: ``True``)
+
+        ================  ===================
+        session           action(s)
+        ================  ===================
+        python session    print and return ``None``
+        Ipython console   return ``DataFrame`` object
+        Jupyter notebook  return ``DataFrame`` object
+        ================  ===================
+
+    reverse
+        *bool* :
+        If ``True``, sort in descending order by ``sortby``.
+        (default: ``True``)
+    since
+        *str* :
+        include runs that started on or after this ISO8601 time
+        (default: ``"1995-01-01"``)
+    sortby
+        *str* :
+        Sort columns by this key, found by exact match in either
+        the ``start`` or ``stop`` document.
+        (default: ``"time"``)
+    tablefmt
+        *str* :
+        When returning an object, specify which type
+        of object to return.
+        (default: ``"dataframe",``)
+
+        ========== ==============
+        value      object
+        ========== ==============
+        dataframe  ``pandas.DataFrame``
+        table      ``str(pyRestTable.Table)``
+        ========== ==============
+
+    timefmt
+        *str* :
+        The ``time`` key (also includes keys ``"start.time"`` and  ``"stop.time"``)
+        will be formatted by the ``self.timefmt`` value.
+        See https://strftime.org/ for examples.  The special ``timefmt="raw"``
+        is used to report time as the raw value (floating point time as used in
+        python's ``time.time()``).
+        (default: ``"%Y-%m-%d %H:%M:%S",``)
+    until
+        *str* :
+        include runs that started before this ISO8601 time
+        (default: ``2100-12-31``)
+    **query
+        *dict* :
+        Any additional keyword arguments will be passed to
+        the databroker to refine the search for matching runs
+        using the ``mongoquery`` package.
+
+    RETURNS
+
+    object:
+        ``None`` or ``str`` or ``pd.DataFrame()`` object
+
+    EXAMPLE::
+
+        TODO
+
+    (new in release 1.5.0)
+    """
+
+    lr = ListRuns(
+        cat=cat,
+        keys=keys,
+        missing=missing,
+        num=num,
+        reverse=reverse,
+        since=since,
+        sortby=sortby,
+        timefmt=timefmt,
+        until=until,
+    )
+
+    table_format_function = dict(
+        dataframe=lr.to_dataframe,
+        table=lr.to_table,
+    ).get(tablefmt or "dataframe", lr.to_table)
+    obj = table_format_function()
+
+    do_print = False
+    if printing:
+        if lr.cat is not None:
+            print(f"catalog: {lr.cat.name}")
+        if printing == "smart":
+            try:
+                get_ipython()  # console or notebook will handle
+            except NameError:
+                do_print = True  # we print it here
+            if tablefmt == "table":
+                do_print = True
+        else:
+            do_print = True
+    if do_print:
+        print(obj)
+        return
+    return obj
+
+
+def listruns_v1_4(
     num=20,
     keys=None,
     printing=True,
@@ -393,6 +781,8 @@ def listruns(
     **db_search_terms,
 ):
     """
+    DEPRECATED: Use newer ``listruns()`` instead.
+
     make a table of the most recent runs (scans)
 
     PARAMETERS
@@ -477,8 +867,13 @@ def listruns(
 
         Out[100]: <pyRestTable.rest_table.Table at 0x7f004e4d4898>
 
-    *new in apstools release 1.1.10*
+    (new in apstools release 1.1.10)
     """
+    warnings.warn(
+        "DEPRECATED: listruns_v1_4() will be removed"
+        " in a future release.  Instead, use newer ``listruns()``.",
+        DeprecationWarning
+    )
     db = getDatabase(db=db, catalog_name=catalog_name)
     keys = keys or []
     num_runs_requested = min(abs(num), len(db))
@@ -494,9 +889,14 @@ def listruns(
         databroker.queries.TimeRange(since=since, until=until)
     ).search(db_search_terms)
 
+    sortKey = "time"
+
+    def sorter(uid):
+        return cat[uid].metadata["start"][sortKey]
+
     table = pyRestTable.Table()
     table.labels = "short_uid   date/time  exit".split() + labels
-    for uid in cat:
+    for uid in sorted(cat, key=sorter, reverse=True):
         if len(table.rows) == num_runs_requested:
             break
         run = cat[uid]
@@ -682,7 +1082,7 @@ def replay(headers, callback=None, sort=True):
         Sort the headers chronologically if True.
         (default:``True``)
 
-    *new in apstools release 1.1.11*
+    (new in apstools release 1.1.11)
     """
     callback = callback or ipython_shell_namespace().get(
         "bec",  # get from IPython shell
@@ -829,7 +1229,7 @@ def listobjects(show_pv=True, printing=True, verbose=False, symbols=None):
 
         In [2]:
 
-    *new in apstools release 1.1.8*
+    (new in apstools release 1.1.8)
     """
     table = pyRestTable.Table()
     table.labels = ["name", "ophyd structure"]
@@ -1094,6 +1494,8 @@ class EmailNotifications(object):
     """
     send email notifications when requested
 
+    .. index:: apstools Utility; EmailNotifications
+
     use default OS mail utility (so no credentials needed)
 
     EXAMPLE
@@ -1149,6 +1551,8 @@ class ExcelDatabaseFileBase(object):
     """
     base class: read-only support for Excel files, treat them like databases
 
+    .. index:: apstools Utility; ExcelDatabaseFileBase
+
     Use this class when creating new, specific spreadsheet support.
 
     EXAMPLE
@@ -1261,6 +1665,7 @@ class ExcelDatabaseFileGeneric(ExcelDatabaseFileBase):
     """
     Generic (read-only) handling of Excel spreadsheet-as-database
 
+    .. index:: apstools Utility; ExcelDatabaseFileGeneric
     .. index:: Excel scan, scan; Excel
 
     .. note:: This is the class to use when reading Excel spreadsheets.
@@ -1691,6 +2096,8 @@ def print_snapshot_list(db, printing=True, **search_criteria):
 
 def json_export(headers, filename, zipfilename=None):
     """
+    DEPRECATED: Use *databroker-pack* package instead.
+
     write a list of headers (from databroker) to a file
 
     PARAMETERS
@@ -1734,6 +2141,11 @@ def json_export(headers, filename, zipfilename=None):
         datasets = json_import("data.json)
 
     """
+    warnings.warn(
+        "DEPRECATED: json_import() will be removed"
+        " in a future release.  Instead, use *databroker-pack* package.",
+        DeprecationWarning
+    )
     datasets = [list(h.documents()) for h in headers]
     buf = json.dumps(datasets, cls=NumpyEncoder, indent=2)
 
@@ -1747,6 +2159,8 @@ def json_export(headers, filename, zipfilename=None):
 
 def json_import(filename, zipfilename=None):
     """
+    DEPRECATED: Use *databroker-pack* package instead.
+
     read the file exported by :func:`~json_export()`
 
     RETURNS
@@ -1772,6 +2186,11 @@ def json_import(filename, zipfilename=None):
                     db.insert(k, doc)
 
     """
+    warnings.warn(
+        "DEPRECATED: json_import() will be removed"
+        " in a future release.  Instead, use *databroker-pack* package.",
+        DeprecationWarning
+    )
     if zipfilename is None:
         with open(filename, "r") as fp:
             buf = fp.read()
@@ -1910,3 +2329,209 @@ def copy_filtered_catalog(source_cat, target_cat, query=None):
         )
         for key, doc in run.documents():
             target_cat.v1.insert(key, doc)
+
+
+_findpv_registry = None
+
+
+class PVRegistry:
+    """
+    Cross-reference EPICS PVs with ophyd EpicsSignalBase objects.
+    """
+
+    def __init__(self, ns=None):
+        """
+        Search ophyd objects for PV or ophyd names.
+
+        The rebuild starts with the IPython console namespace
+        (and defaults to the global namespace if the former
+        cannot be obtained).
+
+        PARAMETERS
+
+        ns *dict* or `None`: namespace dictionary
+        """
+        self._pvdb = defaultdict(lambda: defaultdict(list))
+        self._odb = {}
+        self._device_name = None
+        self._known_device_names = []
+        g = ns or ipython_shell_namespace() or globals()
+
+        # kickoff the registration process
+        logger.debug(
+            "Cross-referencing EPICS PVs with"
+            " Python objects and ophyd symbols"
+        )
+        self._ophyd_epicsobject_walker(g)
+
+    def _ophyd_epicsobject_walker(self, parent):
+        """
+        Walk through the parent object for ophyd Devices & EpicsSignals.
+
+        This function is used to rebuild the ``self._pvdb`` object.
+        """
+        if isinstance(parent, dict):
+            keys = parent.keys()
+        else:
+            keys = parent.component_names
+            _nm_base = self._device_name
+        for k in keys:
+            if isinstance(parent, dict):
+                _nm_base = []
+                v = self._ref_dict(parent, k)
+            else:
+                v = self._ref_object_attribute(parent, k)
+            if v is None:
+                continue
+            # print(k, type(v))
+            if isinstance(v, ophyd.signal.EpicsSignalBase):
+                try:
+                    self._signal_processor(v)
+                    self._odb[v.name] = ".".join(self._device_name + [k])
+                except (KeyError, RuntimeError) as exc:
+                    logger.error(
+                        "Exception while examining key '%s': (%s)",
+                        k, exc
+                    )
+            elif isinstance(v, ophyd.Device):
+                # print("Device", v.name)
+                self._device_name = _nm_base + [k]
+                if v.name not in self._known_device_names:
+                    self._odb[v.name] = ".".join(self._device_name)
+                    self._known_device_names.append(v.name)
+                    self._ophyd_epicsobject_walker(v)
+
+    def _ref_dict(self, parent, key):
+        """Accessor used by ``_ophyd_epicsobject_walker()``"""
+        return parent[key]
+
+    def _ref_object_attribute(self, parent, key):
+        """Accessor used by ``_ophyd_epicsobject_walker()``"""
+        try:
+            obj = getattr(parent, key, None)
+            return obj
+        except (KeyError, RuntimeError, TimeoutError) as exc:
+            logger.error(
+                "Exception while getting object '%s.%s': (%s)",
+                ".".join(self._device_name), key, exc
+            )
+
+    def _register_signal(self, signal, pv, mode):
+        """Register a signal with the given mode."""
+        fdn = full_dotted_name(signal)
+        if fdn not in self._pvdb[pv][mode]:
+            self._pvdb[pv][mode].append(fdn)
+
+    def _signal_processor(self, signal):
+        """Register a signal's read & write PVs."""
+        self._register_signal(signal, signal._read_pv.pvname, "R")
+        if hasattr(signal, "_write_pv"):
+            self._register_signal(signal, signal._write_pv.pvname, "W")
+
+    def search_by_mode(self, pvname, mode="R"):
+        """Search for PV in specified mode."""
+        if mode not in ["R", "W"]:
+            raise ValueError(
+                f"Incorrect mode given ({mode}." "  Must be either `R` or `W`."
+            )
+        return self._pvdb[pvname][mode]
+
+    def search(self, pvname):
+        """Search for PV in both read & write modes."""
+        return dict(
+            read=self.search_by_mode(pvname, "R"),
+            write=self.search_by_mode(pvname, "W"),
+        )
+
+    def ophyd_search(self, oname):
+        """Search for ophyd object by ophyd name."""
+        return self._odb.get(oname)
+
+
+def _get_pv_registry(force_rebuild, ns):
+    """
+    Check if need to build/rebuild the PV registry.
+
+    PARAMETERS
+
+    force_rebuild
+        *bool* :
+        If ``True``, rebuild the internal registry that maps
+        EPICS PV names to ophyd objects.
+    ns
+        *dict* or `None` :
+        Namespace dictionary of Python objects.
+
+    """
+    global _findpv_registry
+    if _findpv_registry is None or force_rebuild:
+        _findpv_registry = PVRegistry(ns=ns)
+    return _findpv_registry
+
+
+def findbyname(oname, force_rebuild=False, ns=None):
+    """
+    Find the ophyd (dotted name) object associated with the given ophyd name.
+
+    PARAMETERS
+
+    oname
+        *str* :
+        ophyd name to search
+    force_rebuild
+        *bool* :
+        If ``True``, rebuild the internal registry that maps
+        ophyd names to ophyd objects.
+    ns
+        *dict* or `None` :
+        Namespace dictionary of Python objects.
+
+    RETURNS
+
+    str or ``None``:
+        Name of the ophyd object.
+
+    EXAMPLE::
+
+        In [45]: findbyname("adsimdet_cam_acquire")
+        Out[45]: 'adsimdet.cam.acquire'
+
+    (new in apstools 1.5.0)
+    """
+    return _get_pv_registry(force_rebuild, ns).ophyd_search(oname)
+
+
+def findbypv(pvname, force_rebuild=False, ns=None):
+    """
+    Find all ophyd objects associated with the given EPICS PV.
+
+    PARAMETERS
+
+    pvname
+        *str* :
+        EPICS PV name to search
+    force_rebuild
+        *bool* :
+        If ``True``, rebuild the internal registry that maps
+        EPICS PV names to ophyd objects.
+    ns
+        *dict* or `None` :
+        Namespace dictionary of Python objects.
+
+    RETURNS
+
+    dict or ``None``:
+        Dictionary of matching ophyd objects, keyed by how the
+        PV is used by the ophyd signal.  The keys are
+        ``read`` and ``write``.
+
+    EXAMPLE::
+
+        In [45]: findbypv("ad:cam1:Acquire")
+        Out[45]: {'read': [], 'write': ['adsimdet.cam.acquire']}
+
+        In [46]: findbypv("ad:cam1:Acquire_RBV")
+        Out[46]: {'read': ['adsimdet.cam.acquire'], 'write': []}
+
+    """
+    return _get_pv_registry(force_rebuild, ns).search(pvname)
